import React from 'react'
import {
    ActivityIndicator,
    Animated,
    AsyncStorage,
    Dimensions,
    Easing,
    ScrollView,
    Text,
    View
} from 'react-native'
import dateFormat from './util'
const {height } = Dimensions.get('window');
import styles from './style'
const DATE_KEY = 'RefreshDate';
const RefreshStatus = {
    pullToRefresh: 0,
    releaseToRefresh: 1,
    refreshing: 2
};
const PaginationStatus = {
    firstLoad: 0,
    waiting: 1,
    allLoaded: 2
};

export default class RefreshableHeard extends ScrollView {
    static defaultProps = {
        horizontal: false,
        scrollEnabled: true,
        header: null,
        refreshable: true,
        refreshableTitlePull: '下拉刷新',
        refreshableTitleRefreshing: '加载中...',
        refreshableTitleRelease: '下拉刷新中...',
        customRefreshView: null,
        displayDate: false,
        dateFormat: 'yyyy-MM-dd hh:mm',
        dateTitle: '刷新时间: ',
        arrowImageSource: require('./downArrow.png'),
        refreshViewStyle: undefined,
        dateStyle: undefined,
        refreshViewHeight: 80,
        insideOfUltimateListView: false
    };

    _offsetY = 0;
    _isRefreshing = false;
    _dragFlag = false;

    constructor(props) {
        super(props);
        this.state = {
            arrowAngle: new Animated.Value(0),
            refreshStatus: RefreshStatus.pullToRefresh,
            refreshTitle: this.props.refreshableTitlePull,
            date: this.props.date
        };
        this.onScroll = this.onScroll.bind(this);
    }

    async componentDidMount() {
        try {
            let result = await AsyncStorage.getItem(DATE_KEY);
            if (result) {
                result = parseInt(result, 10);
                this.setState({
                    date: dateFormat(new Date(result), this.props.dateFormat)
                })
            } else {
                this.setState({
                    date: dateFormat(new Date(), this.props.dateFormat)
                })
            }
        } catch (err) {
            console.log(err)
        }
    }
    //滑动监听
     onScroll(event){
        // console.log('onScroll()');
        const { y } = event.nativeEvent.contentOffset;
        this._offsetY = y;
        if (this._dragFlag) {
            if (!this._isRefreshing) {
                const height = this.props.refreshViewHeight;
                if (y <= -height) {
                    this.setState({
                        refreshStatus: RefreshStatus.releaseToRefresh,
                        refreshTitle: this.props.refreshableTitleRelease
                    });
                    Animated.timing(this.state.arrowAngle, {
                        toValue: 1,
                        duration: 500,
                        easing: Easing.inOut(Easing.quad)
                    }).start()
                } else {
                    this.setState({
                        refreshStatus: RefreshStatus.pullToRefresh,
                        refreshTitle: this.props.refreshableTitlePull
                    });
                    Animated.timing(this.state.arrowAngle, {
                        toValue: 0,
                        duration: 500,
                        easing: Easing.inOut(Easing.quad)
                    }).start()
                }
            }
        }
        if (this.props.onScroll) {
            this.props.onScroll(event)
        }
    };
    //初始拉动的触发
    onScrollBeginDrag = (event) => {
        // console.log('onScrollBeginDrag()');
        this._dragFlag = true;
        this._offsetY = event.nativeEvent.contentOffset.y;
        if (this.props.onScrollBeginDrag) {
            this.props.onScrollBeginDrag(event)
        }
    };
    //结束的触发
    onScrollEndDrag = (event) => {
        // console.log('onScrollEndDrag()');
        this._dragFlag = false;
        const { y } = event.nativeEvent.contentOffset;
        this._offsetY = y;
        const height = this.props.refreshViewHeight;
        if (!this._isRefreshing) {
            if (this.state.refreshStatus === RefreshStatus.releaseToRefresh) {
                this._isRefreshing = true;
                this.setState({
                    refreshStatus: RefreshStatus.refreshing,
                    refreshTitle: this.props.refreshableTitleRefreshing
                });
                this._scrollview.scrollTo({ x: 0, y: -height, animated: true });
                if (this.props.insideOfUltimateListView) {
                    this.props.onRefresh()
                } else {
                    this.props.onRefresh(() => {
                        this.onRefreshEnd()
                    })
                }
            }
        } else if (y <= 0) {
            this._scrollview.scrollTo({ x: 0, y: -height, animated: true })
        }
        if (this.props.onScrollEndDrag) {
            this.props.onScrollEndDrag(event)
        }
    };

    scrollTo = (option) => {
        this._scrollview.scrollTo(option)
    };

    scrollToEnd = (option) => {
        this._scrollview.scrollToEnd(option)
    };
    //刷新结束
    onRefreshEnd = () => {
        // console.log('onRefreshEnd()');
        if (this.state.refreshStatus === RefreshStatus.refreshing) {
            this._isRefreshing = false;
            const now = new Date().getTime();
            this.setState({
                refreshStatus: RefreshStatus.pullToRefresh,
                refreshTitle: this.props.refreshableTitlePull,
                date: dateFormat(now, this.props.dateFormat)
            });
            AsyncStorage.setItem(DATE_KEY, now.toString());
            Animated.timing(this.state.arrowAngle, {
                toValue: 0,
                duration: 500,
                easing: Easing.inOut(Easing.quad)
            }).start();
            this._scrollview.scrollTo({ x: 0, y: 0, animated: true })
        }
    };
    //头部刷新的
    renderRefreshHeader() {
        if (this.props.customRefreshView) {
            return (
                <View style={[styles.header, this.props.refreshViewStyle]}>
                    {this.props.customRefreshView(this.state.refreshStatus, this._offsetY)}
                </View>
            )
        }

        return (
            <View style={[styles.header, this.props.refreshViewStyle]}>
                <View style={styles.status}>
                    {this.renderSpinner()}
                    <Text style={styles.statusTitle}>{this.state.refreshTitle}</Text>
                </View>
                {this.props.displayDate &&
                <Text style={[styles.date, this.props.dateStyle]}>{this.props.dateTitle + this.state.date}</Text>
                }
            </View>
        )
    }
    //刷新动画
    renderSpinner() {
        if (this.state.refreshStatus === RefreshStatus.refreshing) {
            return (
                <ActivityIndicator style={{ marginRight: 10 }} />
            )
        }
        return (
            <Animated.Image
                source={this.props.arrowImageSource}
                resizeMode="contain"
                style={[styles.arrow,
                    this.props.arrowImageStyle,
                    {
                        transform: [{
                            rotateX: this.state.arrowAngle.interpolate({
                                inputRange: [0, 1],
                                outputRange: ['0deg', '-180deg']
                            })
                        }]
                    }]}
            />
        )
    }

    render() {
        return (
            <ScrollView
                ref={c => this._scrollview = c}
                {...this.props}
                scrollEventThrottle={16}
                onScroll={this.onScroll}
                contentContainerStyle={{ minHeight: height }}
                // onMomentumScrollEnd={this.onScrollEndDrag}
                onScrollEndDrag={this.onScrollEndDrag}
                onScrollBeginDrag={this.onScrollBeginDrag}
            >
                {this.renderRefreshHeader()}
                {this.props.children}
            </ScrollView>
        )
    }
}


